package cologne.eck.all_peas.files;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Displays the selected files.
 * Displays selected files, hierarchy, invalid files, file number, all selected size.
 * Contains a ProgressBar to show the encryption progress.
 */


import java.awt.Color;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.IconManager;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.tools.FileTools;
//import cologne.eck.tools.Help;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
public class FileTypePanel extends JPanel implements ActionListener{
	
	/*
	 * instance
	 */
	private FileTypePanel ftp;
	/*
	 * the associated FileComposer for this FileTypePanel
	 */
	private FileComposer fc;// = new FileComposer();	
	/*
	 * Task to show a progress verticalBar 
	 */
	private ProgressTask task;	
	/*
	 * The ResourceBundle for language support
	 */
	private ResourceBundle languageBundle = null;
	/*
	 * JPanel with addButton, used in PeaFactory and in test case 
	 */
	private JPanel buttonPanel;// button to add new files, used in PeaFactory	
	/**
	 * JPanel containing all check boxes of valid file names
	 */
	private JPanel checkPanel; // contains the JCheckBoxes: one for each file
	/**
	 * JPanel containing all check boxes of invalid file names
	 */
	private JPanel invalidCheckPanel; // contains the JCheckBoxes: one for each file
	/**
	 * Button to close or open invalid panel
	 */
	private JButton invalidCloseButton;
	/**
	 * Panel that contains invalid check boxes and label + button to close.
	 * This is only added if invalid files occur
	 */
	private JPanel invalidPanel;
	/**
	 * Scroll pane for invalidCheckPanel. This can be added or removed
	 */
	private JScrollPane invalidScroll;
	/*
	 * show invalid panel (invalid files occurred) or not
	 */
	private boolean showInvalidPanel = false;
	/*
	 * Show file hierarchy or full file names
	 */
	private JCheckBox hierarchyCheckBox;
	/*
	 * Label to display file number and size:
	 * Initialization required before FileTypePanel in fc.initFileNames() 
	 */
	private JLabel infoLabel = new JLabel(); // displays file number and size
	/*
	 * Button to open FileChooser
	 */
	private JButton addNewFilesButton;
	/*
	 * Button to update check boxes
	 */
	JButton updateButton;

	/*
	 * Color to indicate if file is invalid, directory or file
	 */
	private static final Color directoryColor = new Color(215,215,215); // background color of JCheckBox for directories
	private static final Color invalidDirectoryColor = new Color(235,210,210); // background color of JCheckBox for directories
	private static final Color fileColor = new Color(245,245,245);
	private static final Color invalidColor = new Color(255, 220, 220);
	/*
	 * Labels with informations about color meaning
	 */
	private JLabel fileColorLabel;
	private JLabel directoryColorLabel;
	private JLabel invalidFileColorLabel;
	private JLabel invalidDirectoryColorLabel;	
	
	/**
	 * Panel to show selected file names in check boxes.
	 * 
	 * @param scrollWidth	the width of the scroll pane for check box panel
	 * @param scrollHeight	the height of the scroll pane for check box panel
	 * @param _plainModus	true if files are plain text, false for encrypted files
	 * @param addButton		true: add a button to open a FileChooser
	 * @param _fc			the associated FileComposer with this instance
	 * @param window		the parent window of this panel
	 */

	public FileTypePanel(int scrollWidth, int scrollHeight, 
			boolean _plainModus, boolean addButton, 
			FileComposer _fc, Object window){

		languageBundle = PeaProperties.getBundle();
		fc = _fc;
		fc.setPlainModus(_plainModus);
		fc.setFileTypePanel(this);
		
		ftp = this;
		if (_plainModus == true) {
			PswDialogView.setUI(false);
		} else {
			PswDialogView.setUI(true);
		} 
		int fontSize = PswDialogView.getFontsize();
		
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// button to open FileChooser
		if (addButton == true){ // not in File PEA but in PeaFactory
			buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
			buttonPanel.setBorder(new EmptyBorder(10,0,5,0));
			addNewFilesButton = new JButton(languageBundle.getString("add_new_file"));
			addNewFilesButton.addActionListener(this);
			addNewFilesButton.setActionCommand("addFile");
			buttonPanel.add(addNewFilesButton);
			buttonPanel.add(Box.createHorizontalGlue());
			this.add(buttonPanel);	
		}
		String status = null;
		ImageIcon statusIcon = null;
		if (_plainModus == true){
			status = languageBundle.getString("status_unencrypted");
			statusIcon = IconManager.loadIcon("unlocked.png", languageBundle.getString("open_padlock"), 
					null, languageBundle.getString("unencrypted_indicator") );
		} else {
			status = languageBundle.getString("status_encrypted");
			statusIcon = IconManager.loadIcon("locked.png", languageBundle.getString("closed_padlock"), 
					null, languageBundle.getString("encrypted_indicator") );
		}
		JPanel statusPanel= new JPanel();
		statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
		statusPanel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 30));
		if (statusIcon != null) {
			statusPanel.add(Box.createHorizontalGlue());
			statusPanel.add(new JLabel(statusIcon));
			statusPanel.add(Box.createHorizontalStrut(10));
		}		
		JLabel statusLabel= new JLabel(status);
		statusLabel.setFont(new Font(Font.MONOSPACED, Font.ITALIC, fontSize));
		statusPanel.add(statusLabel);
		this.add(statusPanel);
	
		checkPanel = new JPanel(); // the panel that holds the check boxes
		checkPanel.setLayout(new BoxLayout(checkPanel, BoxLayout.Y_AXIS));	
		if (fc.getPlainModus() == false) { // check if there are files to show
			fc.initFileNames();
		}		

		JScrollPane scroll = new JScrollPane(checkPanel);
		scroll.setPreferredSize(new Dimension(scrollWidth, scrollHeight));		
		this.add(scroll);		
		
		JPanel infoPanel = new JPanel();// Panel to show number and size of the files
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
		infoPanel.setBorder(new EmptyBorder(0,0,10,0));

		infoLabel.setText(languageBundle.getString("number_of_files") 
				+ fc.getFileModel().getFileNumber() 
				+ ", " + languageBundle.getString("overall_size")
				+ fc.getFileModel().getAllSize() + " - (MiB): " 
				+ fc.getFileModel().getAllSize()/1024/1024);
		infoPanel.add(infoLabel);
		infoPanel.add(Box.createHorizontalGlue());
		this.add(infoPanel);
		//this.add(Box.createVerticalStrut(10));
		
		invalidPanel = new JPanel();
		invalidPanel.setLayout(new BoxLayout(invalidPanel, BoxLayout.PAGE_AXIS));
		invalidPanel.setBorder(new LineBorder(Color.GRAY, 1));
		
		JPanel invalidButtonPanel = new JPanel();
		invalidButtonPanel.setLayout(new BoxLayout(invalidButtonPanel, BoxLayout.LINE_AXIS));
		invalidButtonPanel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 30));
		JLabel invalidLabel = new JLabel("Invalid files:");
		invalidLabel.setOpaque(true);
		invalidLabel.setBackground(Color.LIGHT_GRAY);
//		invalidLabel.setBorder(new LineBorder(Color.BLACK, 1));
		invalidButtonPanel.add(Box.createHorizontalStrut(5));
		invalidButtonPanel.add(invalidLabel);
		invalidButtonPanel.add(Box.createHorizontalGlue());
		invalidCloseButton = new JButton("-");		
		//invalidCloseButton.setOpaque(true);
		//invalidCloseButton.setBackground(Color.WHITE);
		//invalidCloseButton.setBorder(new LineBorder(Color.BLACK, 1));
		invalidCloseButton.setFont(new Font(Font.MONOSPACED , Font.PLAIN, fontSize - 1));
		//invalidCloseButton.setMargin(new Insets(0,0,0,0));
		//invalidCloseButton.setMinimumSize(new Dimension(30,30));
		invalidCloseButton.addActionListener(this);
		invalidCloseButton.setActionCommand("invalidCheckPanel");
		invalidButtonPanel.add(invalidCloseButton);
		invalidButtonPanel.add(Box.createHorizontalStrut(5));
		invalidPanel.add(invalidButtonPanel);

		
		invalidCheckPanel = new JPanel(); // the panel that holds the check boxes
		invalidCheckPanel.setLayout(new BoxLayout(invalidCheckPanel, BoxLayout.Y_AXIS));	
		if (fc.getPlainModus() == false) { // check if there are files to show
			fc.initFileNames();
		}		
//		invalidCheckPanel.setMinimumSize(new Dimension(300, 30));
//		invalidCheckPanel.setMaximumSize(new Dimension((int)PswDialogView.getScreenwidth(), 100));

		invalidScroll = new JScrollPane(invalidCheckPanel);
		invalidScroll.setPreferredSize(new Dimension(scrollWidth, 90));		
		invalidPanel.add(invalidScroll);		
	}
	
	 // just to test the view	 
/*	public static void main(String[] args) {

		JFrame frame = new JFrame();
		FileTypePanel ftp = new FileTypePanel(300,300, true, true,
				new FileComposer(new FileModel()), frame) ;
		PeaProperties p = new PeaProperties();
		p.setFileType("file");
		PswDialogView.setUI(false);
		p.setTypePanel(ftp);
		p.setWorkingMode("-t");
		FileControl control = FileControl.getInstance();
		//FileTypePanel.printErrors = true;
		frame.add(ftp );
		frame.setLocation(100,100);
		frame.pack();
		frame.setVisible(true);		
	} */


	@Override
	public void actionPerformed(ActionEvent ape) {
		
		String command = ape.getActionCommand();

		if (command.equals("addFile")) {		
			boolean showHiddenFilesOption = false;
			if (PeaProperties.getFileType().equals("file")){
				showHiddenFilesOption = true;
			}
			fc.addAction(this,
					PeaProperties.getBundle().getString("select_files_to_encrypt"),
					showHiddenFilesOption, false );
		    updateWindow();
		    infoLabel.setText(fc.getCurrentNumberAndSize());
		    
		} else if (command.equals("invalidCheckPanel")) {	
			
			JButton button = (JButton) ape.getSource();
			String text = button.getText();
			
			if (text.equals("+")){
				button.setText("-");
				if ( invalidScroll.getParent() != invalidPanel) {
					invalidPanel.add(invalidScroll);
				} 
			} else if (text.equals("-")){

				invalidPanel.remove(invalidScroll);
				button.setText("+");				
			}

			invalidCheckPanel.invalidate();
			invalidCheckPanel.validate();
			invalidPanel.invalidate();
			invalidPanel.validate();
			updateWindow();
		    
		} else if (command.equals("hierarchy")) {	

			if (hierarchyCheckBox.isSelected()){

				String[] newTexts = fc.showFileHierarchy(getAllFileNames());
				// set the new texts as check box texts
				updateCheckBoxTexts(newTexts);
				checkPanel.repaint();
			} else {
				showFullFileNames();
			}
			updateWindow();
       	
		} else if (command.equals("dirSelection")) { // non empty directory			
			// get the selected/deselected checkBox:
			JCheckBox dirCheck = (JCheckBox) ape.getSource();				
			// check if dirCheck is selected or not 
			boolean select;
			if (dirCheck.isSelected() == true) {
				select = true;
			} else {
				select = false;
			} 
			// select/deselect all children of this directory and check
			// if at least the staus of one check box was changed
			boolean stateChanged = selectChildren(select, dirCheck);
			
			// if deselect: deselect all parent directories
			if (select == false && stateChanged == true) {
				deselectParentDirectories(dirCheck);				
			}
			infoLabel.setText(fc.getCurrentNumberAndSize());

		} else if (command.equals("fileSelection")) {					
			// get the associated file model
			FileModel model = fc.getFileModel();
			// get the check box
			JCheckBox fileCheck = (JCheckBox) ape.getSource();
			// get the file name of the check box
			String fileName = fileCheck.getName();
			// get the file size of this file and update allSize
			long fileSize = FileTools.getFileSize(fileName);			
			if (fileCheck.isSelected() == true) {
				model.setAllSize(model.getAllSize() + fileSize);
				model.setFileNumber(model.getFileNumber() + 1);
				// select parent directories if became full
				selectFullParentDirectories(fileCheck);
			} else {		
				model.setAllSize(model.getAllSize() - fileSize);
				model.setFileNumber(model.getFileNumber() - 1);
				// deselect all parent directories
				deselectParentDirectories(fileCheck);
			}
			infoLabel.setText(fc.getCurrentNumberAndSize());			
		} 
	}
	
	//==============================================================================================
	// Internal helper functions
	
	/**
	 * Get the index of a check box in the checkPanel. 
	 * This function iterates over all entries.
	 * 
	 * @param check		the check box
	 * 
	 * @return			the index of the check box or
	 * 					-1 if the check box is not included
	 */
	private final int getComponentIndex(JCheckBox check) {
		if (check != null ) {
			int len = checkPanel.getComponentCount();
			for (int i = 0; i < len; i++) {
				if (checkPanel.getComponent(i) == check){
					return i;
				}
			}
		}
		return -1;
	}
	/**
	 * Show the full name of a file. This method just
	 * takes the current file names in the panel.
	 */
	public final void showFullFileNames(){
		
		int size = checkPanel.getComponentCount();
		for (int i = 0; i < size; i++) {
			JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
			check.setText(check.getName());
		}
	}
	/**
	 * Cleanup file view: remove unselected and
	 * invalid files and check valid, selected files again
	 */
	public void cleanupView() {
		// get only selected file names
		String[] selectedFileNames = getValidFileNames(true);
		if (selectedFileNames == null) {
			return;
		}
		Arrays.sort(selectedFileNames);
		FileModel model = fc.getFileModel();
		// delete all current files
		model.resetTreeMap();
		model.setFileNumber(0);
		model.setAllSize(0);
		// create new map:
		ArrayList<String> list = new ArrayList<String>(Arrays.asList(selectedFileNames));
    	fc.addFilesToMap(list, false, false, false);// false: include sub-directories
    										// false: do not check already added files again
    										// true: add check boxes
//    	for(int i=0;i<model.getAllFiles().length;i++) System.out.println(model.getAllFiles()[i].getAbsolutePath());
    	fc.updateFiles(false);// do not check already listed files again
    	// clear the view:
    	invalidCloseButton.setText("+");    
    	invalidCheckPanel.removeAll();
		invalidCheckPanel.invalidate();
		invalidCheckPanel.validate();
		if (invalidScroll.getParent() == invalidPanel){
			invalidPanel.remove(invalidScroll);
		}
		invalidPanel.repaint();
		updateWindow();
		model.calculateNumberAndSize();
		infoLabel.setText(fc.getCurrentNumberAndSize());
	}
	/**
	 * Select or deselect all children of a directory
	 * check box. Only check boxes among the given
	 * check box are affected. 
	 * 
	 * @param select	true for select, false for deselect
	 * @param check		the directory check box
	 * 
	 * @return			true if the status of at least one check box
	 * 					was modified
	 */
	private boolean selectChildren(boolean select, JCheckBox check){
		
		if (check == null) {
			new UnexpectedValueException("check", "JCheckBox", "is null").printDescription();
			return false;
		}
		if (! check.getBackground().equals(directoryColor)){ // invalid or file
			new UnexpectedValueException("check", "JCheckBox", "has not valid directory color").printDescription();
			return false;
		}		
		boolean stateChanged = false; // check if at least one check box is selected/deselected
		FileModel model = fc.getFileModel();
		String directoryName = check.getName();
		int checkIndex = getComponentIndex(check);// index of this check box in checkPanel
		if (checkIndex == -1) {
			new UnexpectedValueException("checkIndex", "int", "is -1, check box missing").printDescription();
			return false;
		}
		int componentNumber = checkPanel.getComponentCount();
		// select/deselect check boxes among the given check box
		for (int i = checkIndex; i < componentNumber; i++) {
			// get check box among
			JCheckBox nextCheck = (JCheckBox) checkPanel.getComponent(i);
			if (nextCheck.getName().contains(directoryName)){				
				if (nextCheck.getBackground().equals(fileColor)){
					if (select == true && nextCheck.isSelected() == false) {
						stateChanged = true;					
						model.setAllSize(model.getAllSize() // update size
								+ FileTools.getFileSize(nextCheck.getName()));
						model.setFileNumber(model.getFileNumber() + 1); // update number
					} else if (select == false && nextCheck.isSelected() == true) {
						stateChanged = true;
						model.setAllSize(model.getAllSize() // update size
								- FileTools.getFileSize(nextCheck.getName()));	
						model.setFileNumber(model.getFileNumber() - 1); // update number
					}
					nextCheck.setSelected(select); // select/deselct
					
				} else if (nextCheck.getBackground().equals(directoryColor)){
					nextCheck.setSelected(select);
				}				
			} else {
				// file of another directory was reached
				return stateChanged;
			}
		}
		return stateChanged;
	}
	/**
	 * Deselect check boxes of parent directories of a file
	 * if the file was deselected. Only check boxes of
	 * directories above the check box are affected. 
	 * 
	 * @param check	the file check box that was deselected
	 */
	private void deselectParentDirectories(JCheckBox check){
		
		if (check == null) {
			new UnexpectedValueException("check", "JCheckBox", "is null").printDescription();
		}

		// get file name of check box
		String fileName = check.getName();
		// get index of check box in checkPanel
		int checkIndex = getComponentIndex(check);
		if (checkIndex == -1) {
			new UnexpectedValueException("checkIndex", "int", "is -1, check box missing").printDescription();
			return;
		}
		// get name of the parent directory
		String rootDirName = fileName.substring(0, fileName.lastIndexOf(File.separator));//secondSeparator);

		// proof only check boxes above
		for (int i = checkIndex -1;i >= 0; i--) {
			// get the next check box above:
			JCheckBox nextCheck = (JCheckBox) checkPanel.getComponent(i);
			if (nextCheck.getBackground().equals(directoryColor)){ // ignore files and invalid directories
				
				if ( rootDirName.equals(nextCheck.getName()) ){
					// deselect the directory check box
					nextCheck.setSelected(false);
					// update directory to check: parent of current directory
					rootDirName = rootDirName.substring(0, rootDirName.lastIndexOf(File.separator));
				} else {
					if  ( ! nextCheck.getName().contains(rootDirName)){
						// reached another directory... 
						return;
					}
				}
			}	
		}
	}
	/**
	 * If a file check box was selected, this function
	 * checks if parent directories is complete (all
	 * shown children) selects them in this case.
	 * 
	 * @param check		the selected file check box
	 */
	private void selectFullParentDirectories(JCheckBox check){
		String fileName = check.getName();
		int checkIndex = getComponentIndex(check);
		if (checkIndex == -1) {
			new UnexpectedValueException("checkIndex", "int", "is -1, check box missing").printDescription();
			return;
		}
		// get name of the parent directory, this variable will change
		String rootDirName = fileName.substring(0, fileName.lastIndexOf(File.separator));//secondSeparator);

		int lastCheckedIndex = checkIndex;
		// search for unselected parent directories 
		// check if these are complete now and select in this case
		// proof only check boxes above
		for (int i = checkIndex -1;i >= 0; i--) {
			// get the next check box above:
			JCheckBox nextCheck = (JCheckBox) checkPanel.getComponent(i);
			if (nextCheck.getBackground().equals(directoryColor)){ // ignore invalid check boxes
				
				String dirName = nextCheck.getName();
				if ( rootDirName.equals(dirName) ){ // next parent found
					// update directory to check: parent of current directory
					rootDirName = rootDirName.substring(0, rootDirName.lastIndexOf(File.separator));
					
					if (nextCheck.isSelected() == false) {
						// check if now all children are included:
						int completenessResult = checkDirectoryCompleteness(
								nextCheck, dirName, lastCheckedIndex);
						if (completenessResult >= 0) { // is complete
							// select complete directory
							nextCheck.setSelected(true);
							// update last checked index: 
							// next check can begin there
							lastCheckedIndex = completenessResult;
						} else { // deselected check box was found (or failure)
							// no parent of this directory can be complete, so break
							return;
						}
					}
				} else {
					if  ( ! nextCheck.getName().contains(rootDirName)){
						// reached another directory... 
						return;
					}
				}				
			} else if (nextCheck.getBackground().equals(fileColor)){
				if (nextCheck.isSelected() == false){
					// directories are not full because of this file
					return;
				}				
			}
		}
	}
	/**
	 * Check if a directory check box includes all
	 * shown children
	 * 
	 * @param dirCheck		directory check box to check
	 * @param directoryName	name of the directory
	 * @param index			index of selected file check box:
	 * 						check only below this index
	 * 
	 * @return		the index of the last check box that
	 * 				was checked by this call or -1 if the
	 * 				directory is not complete (and failure)		
	 */
	private int checkDirectoryCompleteness(JCheckBox dirCheck, 
			String directoryName, int index){
		
		if (dirCheck == null) {
			new UnexpectedValueException("check", "JCheckBox", "is null").printDescription();
			return -1;
		}
		if (! dirCheck.getBackground().equals(directoryColor)){ // invalid or file
			new UnexpectedValueException("check", "JCheckBox", "is not valid directory color").printDescription();
			return -1;
		}		
		if (index < 0) {
			new UnexpectedValueException("index", "int", "is negative, check box missing").printDescription();
			return -1;
		}
		int componentNumber = checkPanel.getComponentCount();
		// check check boxes among the given check box
		for (int i = index; i < componentNumber; i++) {
			// get check box below
			JCheckBox nextCheck = (JCheckBox) checkPanel.getComponent(i);
			// check for deselected files or directories:
			if (nextCheck.getName().contains(directoryName)){				
				if (nextCheck.getBackground().equals(fileColor)
						|| nextCheck.getBackground().equals(directoryColor)){
					if (nextCheck.isSelected() == false) {
						// not selected check box found: break search
						return -1;
					} 					
				} 			
			} else {
				// file of another directory was reached:
				// no deselected child found
				return i; // index of last checked check box
			}
		}
		// no deselected child found
		return componentNumber;
	}
	/**
	 * Expensive helper function to find the
	 * index of the JCheckBox associated with a file name. 
	 * 
	 * @param fileName	the file name
	 * @return			the index of the check box
	 * 					for this file name or -1 if not found
	 */
/*	private int getIndexOfCheckBox(String fileName) {
		int len = checkPanel.getComponentCount();
		// iterate the checkPanel to find the file name
		for (int i = 0; i < len; i++) {
			String checkName = ((JCheckBox) checkPanel.getComponent(i)).getName();
			if (checkName.equals(fileName)){
				return i;
			}
		}
		if (fc.isPrintErrors() == true) {
			System.err.println("File name not found in check box: " + fileName);
		}
		return -1; // not found
	} */

    /**
     * Set new text for all check boxes. 
     * Note: the check boxes must have been sorted
     * before, the new texts must have the same order
     * 
     * @param newTexts	the new text for the check boxes
     */
    public void updateCheckBoxTexts(String[] newTexts){
    	// number of all check boxes 
    	int checkNumber = checkPanel.getComponentCount();
    	if (checkNumber != newTexts.length) {
    		new UnexpectedValueException("newTexts", "String[]", 
    				"is not equal to number of check boxes").printDescription();
    	}
    	for (int i = 0; i < checkNumber; i++) {
    		JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
    		// check if the order is the same:
    		String checkBoxFileName = check.getName().substring(check.getName().lastIndexOf(File.separator), check.getName().length());
    		String fileName = newTexts[i].substring(newTexts[i].lastIndexOf(File.separator), newTexts[i].length());
    		if (! checkBoxFileName.equals(fileName)){
        		new UnexpectedValueException("newTexts[i]", "String", 
        				"is not equal to check box file name: "
        		+ newTexts[i] + ", " + check.getText()).printDescription();
        		return;
    		}
    		check.setText(newTexts[i]);
    	}
    	checkPanel.repaint();
    }

    /**
     * Get the all file names (selected, unselected,valid, invalid), 
     * as stored in TreeMap
     * 
     * @return	the actually selected file names as an array of strings
     */
    public final String[] getAllFileNames() {
    	
    	// number of all check boxes 
    	int validCheckNumber = checkPanel.getComponentCount();
    	String[] allValidFileNames = new String[validCheckNumber];
    	for (int i = 0; i < validCheckNumber; i++) {
    		JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
			allValidFileNames[i] = check.getName();		
    	}   	
    	int invalidCheckNumber = invalidCheckPanel.getComponentCount();
    	// the maximum array of selected valid file names
    	String[] allInvalidFileNames = new String[invalidCheckNumber];
    	for (int i = 0; i < invalidCheckNumber; i++) {
    		JCheckBox check = (JCheckBox) invalidCheckPanel.getComponent(i);
			allInvalidFileNames[i] = check.getName();		
    	} 
    	String[] allFileNames = new String[allValidFileNames.length
    	                                   + allInvalidFileNames.length];
    	System.arraycopy(allValidFileNames,  0,  
    			allFileNames,  0,  allValidFileNames.length);
    	System.arraycopy(allInvalidFileNames,  0,  
    			allFileNames,  allValidFileNames.length, allInvalidFileNames.length);
    	return allFileNames;
    }

    /**
     * Mark all unselected valid files
     * with the marker "-" at the
     * beginning of the annotation. 
     */
    public void markUnselectedValidFiles(){
    	FileModel model = fc.getFileModel();
    	// number of all check boxes 
    	int validCheckNumber = checkPanel.getComponentCount();
    	//String[] allValidFileNames = new String[validCheckNumber];
    	for (int i = 0; i < validCheckNumber; i++) {
    		JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
    		File file = new File(check.getName());
    		String annotation = model.getAnnotation(file);
    		if (check.isSelected()){
    			model.setAnnotation(file,  annotation);
    		} else {
    			model.setAnnotation(file, "-" + annotation);
    		}	
    	}  
    }
    
	/**
	 *  Updates the root Window (JFrame, JDialog...) 
	 *  to show new files
	 */
	public final void updateWindow() {
		// get the root frame
    	Window frame = (Window) SwingUtilities.getRoot(this);
    	if (frame == null) { // this happens at start  if path file is set
	    	checkPanel.invalidate();
	    	checkPanel.validate();
    		return;
    	}
    	// get current size:
    	Dimension dim = frame.getSize();

    	if (showInvalidPanel == false){
    		//System.out.println("add invalid panel");
    		if (invalidCheckPanel.getComponentCount() > 0){
    			showInvalidPanel = true;
    			//invalidCheckPanel.setVisible(true);

    			this.add(invalidPanel);
    			invalidPanel.invalidate();
    			invalidPanel.validate();

    			this.invalidate();
    			this.validate();
    			
    			frame.invalidate();
    			frame.validate();
    			
    			// check if panel was larger than pack size
    			if (frame.getPreferredSize().getWidth() >= dim.width
    					|| frame.getPreferredSize().getHeight() > dim.height){
    				frame.pack();    				
    			} else {    				
    				frame.setSize(dim);
    			}
    	    	return;    	    	
    		}
    	} else {
			invalidCheckPanel.invalidate();
			invalidCheckPanel.validate();
    	}

    	checkPanel.invalidate();
    	checkPanel.validate();

    	//frame.pack();
    	frame.setSize(dim);
//    	infoLabel.setText(fc.getCurrentNumberAndSize());
    }
    
    //==================== public methods ================================================

	/**
	 * Update the info label
	 */
	public void updateNumberAndSize(){
		infoLabel.setText(fc.getCurrentNumberAndSize());
	}
	
    /**
     * Add a check box for an invalid file to the checkPanel. 
     * The background color of the check box indicates invalidity. 
     * 
     * @param fileName			the file name associated with this check box
     * @param annotation		the annotation for this file
     */
    public final void addInvalidFileCheckBox(
    		String fileName, String annotation) { 
    	JCheckBox newBox = new JCheckBox(annotation + fileName);
    	newBox.setName(fileName);
    	newBox.setOpaque(true);
		newBox.setSelected(false);
		newBox.setEnabled(false);
		newBox.setBackground(invalidColor);
		invalidCheckPanel.add(newBox);
    }
    /**
     * Add a check box for a valid file to the checkPanel. 
     * The background color of the check box indicates  validity. 
     * 
     * @param fileName			the file name associated with this check box
     * @param annotation		the annotation for this file
     */
    public final void addValidFileCheckBox(
    		String fileName, String annotation) { 
    	JCheckBox newBox = new JCheckBox();
    	//System.out.println("annotation: " +annotation);
    	if (annotation.startsWith("-")){
    		// check box was deselected before
    		annotation = annotation.substring(1,annotation.length());
    		fc.getFileModel().setAnnotation(new File(fileName), annotation);
    		newBox.setSelected(false);
    	} else {
    		newBox.setSelected(true);
    	}
    	newBox.setText(annotation + fileName);
    	newBox.setName(fileName);
    	newBox.setOpaque(true);
		newBox.addActionListener(this);
		newBox.setActionCommand("fileSelection");
		
		newBox.setBackground(fileColor);	
		checkPanel.add(newBox);
    }
    /**
     * Add a check box for a valid or invalid directory to the checkPanel. 
     * The background color of the check box indicates validity or invalidity.
     * 
     * @param fileName		the file name associated with this check box
     * @param annotation	the annotation for this file
     * @param valid			true if the directory is valid, false otherwise
     */
    public final void addDirectoryCheckBox(
    		String fileName, String annotation, 
    		boolean valid){
    	JCheckBox newBox = new JCheckBox();
    	newBox.setName(fileName);
		newBox.setOpaque(true);

    	if (valid == true) {
    		if (annotation.startsWith("-")){
    			// check box was deselected before
    			annotation = annotation.substring(1, annotation.length());
    			fc.getFileModel().setAnnotation(new File(fileName), annotation);
    			newBox.setSelected(false);
    		} else {
    			newBox.setSelected(true);
    		}
    		newBox.setBorder(new LineBorder(directoryColor));
    		newBox.setBorderPainted(true);
    		newBox.setBorderPaintedFlat(true);
    		newBox.setBackground(directoryColor);       
    		newBox.addActionListener(this);
    		newBox.setActionCommand("dirSelection");
    		newBox.setText(annotation + fileName);
    		checkPanel.add(newBox);
    		
    	} else { // invalid
    		newBox.setSelected(false);
			newBox.setEnabled(false);
			newBox.setBackground(invalidDirectoryColor);
			newBox.setText(annotation + fileName);
			
   	    	invalidCheckPanel.add(newBox);
    	}
    }
    /**
     * Update the view of files. This method should always
     * been called after check boxes are added or removed
     */
    public void updateFileView(){
    	if (showInvalidPanel == true){
    		invalidCheckPanel.invalidate();
    		invalidCheckPanel.validate();
    	} else {
    		//check if new invalid files occurred
    		if (invalidCheckPanel.getComponentCount() > 0) {
    			updateWindow();
    		}
    	}
    	checkPanel.invalidate();
    	checkPanel.validate();    	
    	infoLabel.setText(fc.getCurrentNumberAndSize());
    }
    /**
     * Get the shown files which are valid but not selected.
     * This is used to display a warning message when the File PEA
     * is closed and some files remain unencrypted
     * 
     * @return	the unselected valid file names
     * 			as an array of Strings
     */
    public final String[] getUnselectedValidFileNames() {
    	
    	// number of all check boxes 
    	int checkNumber = checkPanel.getComponentCount();
    	if (checkNumber == 0) {
    		System.err.println("no files displayed");
    		return null;
    	}
    	ArrayList<String> list = new ArrayList<String>();

    	for (int i = 0; i < checkNumber; i++) {
    		JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
    		if (check.isEnabled() == true	// no invalid files
    				&& check.isSelected() == false	// no selected files
    				&& check.getBackground().equals(fileColor)) { // no directories
    			list.add(check.getName());
    		} 
    	}   	
    	if (list.isEmpty()){
    		return null;
    	} else {
    		// create the return value:
    		String[] result = new String[list.size()];
    		return list.toArray(result);
    	}
    }

    /**
     * Get the actual selected file names, as stored in TreeMap
     * 
     * @param selectedOnly	true: only selected files, 
     * 						false: all valid files
     * 
     * @return	the actually selected file names as an array of strings
     */
    public final String[] getValidFileNames(boolean selectedOnly) {

    	// number of all check boxes 
    	int checkNumber = checkPanel.getComponentCount();
    	if (checkNumber == 0) {
    		//System.err.println("no files displayed");
    		return null;
    	}
    	ArrayList<String> list = new ArrayList<String>();

    	for (int i = 0; i < checkNumber; i++) {
    		JCheckBox check = (JCheckBox) checkPanel.getComponent(i);
    		if (selectedOnly == true){
    			if (check.isSelected() == true) {
    				list.add(check.getName());
    			} 
    		} else {
    			list.add(check.getName());
    		}
    	}   	
    	if (list.isEmpty()){
    		System.err.println("no file found");
    		return null;
    	} else {
    		// create the return value:
    		String[] result = new String[list.size()];
    		return list.toArray(result);
    	}
    }
	/**
	 * Set an already shown check box invalid
	 * 
	 * @param fileName			the file name stored in the map of the model
	 * @param newAnnotation		the new annotation for the file
	 */
/*	public final void setCheckBoxInvalid(String fileName, String newAnnotation) {
		// untested
		File file = new File(fileName);
		FileModel model = fc.getFileModel();

		// get the index of the check box in checkPanel
		int index = getIndexOfCheckBox(fileName);

		if (index < 0) { // file name not found
			if (fc.isPrintErrors() == true) {
				new UnexpectedValueException("index", "int", 
						"is < 0, file name not found in checkPanel: "
								+ fileName).printDescription();
			}
		} 
		String annotation = model.getAnnotation(file);


		JCheckBox check = (JCheckBox) checkPanel.getComponent(index);
		check.setText(annotation + fileName);
		check.setSelected(false);
		check.setEnabled(false);

		if (file.isFile()){    		
			check.setBackground(invalidColor);
			check.setEnabled(false);
			if (check.getActionListeners() != null) {
				check.removeActionListener(this);
			}
			// update number and size
			model.setAllSize(model.getAllSize() - FileTools.getFileSize(fileName) );
			model.setFileNumber(model.getFileNumber() - 1);

		} else if (file.isDirectory()){    		
			check.setBackground(invalidDirectoryColor);
			check.setEnabled(false);
			if (check.getActionListeners() != null) {
				check.removeActionListener(this);
			}
		} else {
			// nothing to do, already marked as invalid
		}
		checkPanel.remove(check);
		invalidCheckPanel.add(check);
		//System.out.println(check.getName());
		
		updateWindow();
	} */
	
	public void removeAllCheckBoxes(){
		checkPanel.removeAll();
		if (invalidCheckPanel != null) {
			invalidCheckPanel.removeAll();
		}
	}
 
     
    //========================================================================================
    // Getter & Setter
    
    /**
     * Get the FileComposer associated with this 
     * instance of FileTypePanel
     * 
     * @return	the FileComposer for this FileTypePanel
     */
    public final FileComposer getFileComposer(){
    	return fc;
    }
    /**
     * Set the FileComposer associated with this 
     * instance of FileTypePanel
     * 
     * @param 	the new FileComposer to set
     */
    public final void setFileComposer(FileComposer _fc) {
    	fc = _fc;
    } 
    /**
     * Remove the button to open a FileChooser
     */
    public final void removeAddButton() {
    	if (buttonPanel != null) {
    		this.remove(buttonPanel);
        	this.validate();
        	this.invalidate();
    	}
    }
    /**
     * Set the language for this FileTypePanel.
     * 
     * @param newBundle		the ResourceBundle for the language to set
     */
    public void setLanguage( ResourceBundle newBundle){
    	languageBundle = newBundle;
		infoLabel.setText(languageBundle.getString("number_of_files") 
				+ fc.getFileModel().getFileNumber() 
				+ ", " + languageBundle.getString("overall_size")
				+ fc.getFileModel().getAllSize() + " - (MiB): " 
				+ fc.getFileModel().getAllSize()/1024/1024);
		// update components:
		addNewFilesButton.setText(languageBundle.getString("add_new_file"));				
		fileColorLabel.setText(languageBundle.getString("valid_file"));
		directoryColorLabel.setText(languageBundle.getString("valid_directory"));
		invalidFileColorLabel.setText(languageBundle.getString("invalid_file"));
		invalidDirectoryColorLabel.setText(languageBundle.getString("invalid_empty_directory"));
		updateButton.setText(languageBundle.getString("update"));
		updateButton.setToolTipText(languageBundle.getString("tooltip_update"));
		hierarchyCheckBox.setText(languageBundle.getString("tree_view"));
		hierarchyCheckBox.setToolTipText(languageBundle.getString("tooltip_tree_view"));
    }


	//================================================
	// ProgrssBar to show progress: methods and  inner class
    /**
     * Start the progress verticalBar
     */
	public final void startProgressTask(){
		task = new ProgressTask();
		task.execute();
	}	
	/**
	 * Update the progress verticalBar
	 * 
	 * @param value		the new value to show
	 */
	public final void setProgressValue(int value){
		if (task != null) {
			task.setValue(value);
		}
	} 
	/**
	 * Close the progress verticalBar
	 */
	public final void closeProgressTask(){
		if (task != null) {
			task.close();
		}
	} 

	/**
	 * Progress verticalBar to display the encryption/decryption progress
	 * for many or large files
	 */
	public class ProgressTask extends SwingWorker<Void, Void> {
		// TODO extra class
    	JProgressBar threadProgressBar;
    	
    	JDialog dialog;
    	
    	// Show whether the process is finished: (set in close)
    	boolean processFinished = false;

        @Override
        public Void doInBackground() {
        	
    		//System.out.println(" eventThread: " + SwingUtilities.isEventDispatchThread() );
        	try { //wait 0.5s
        		Thread.sleep(500);
        	} catch (InterruptedException e) {
        		System.err.println("progressTask: thread interrupted " + e.toString());
        		return null;
        	}
        	// check if process is done:
        	if (processFinished == false) {
               	String modusText = ""; // used in title of dialog
            	if (ftp.getFileComposer().getPlainModus() == true) {
            		modusText = PeaProperties.getBundle().getString("encryption");//"encryption";
            	} else {
            		modusText = PeaProperties.getBundle().getString("decryption");
            	}

            	// create progress verticalBar: 
            	//threadProgressBar = new JProgressBar();
        		threadProgressBar = new JProgressBar(0,100);
        		threadProgressBar.setValue(1);
        		threadProgressBar.setStringPainted(true);
        		threadProgressBar.setPreferredSize(new Dimension(400, 30));        		
            	
            	JPanel contentPane = new JPanel();
            	contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
            	contentPane.add(threadProgressBar);
            	
            	JOptionPane pane = new JOptionPane( contentPane,//threadProgressBar, 
            	   JOptionPane.PLAIN_MESSAGE, 
            	    JOptionPane.DEFAULT_OPTION);

            	dialog = pane.createDialog(ftp, 
            			PeaProperties.getBundle().getString("file") + " "
            			+ modusText + " "
            			+ PeaProperties.getBundle().getString("in_progress"));// title

            	dialog.setCursor(CursorManager.getWaitCursor());
            	dialog.pack();
            	if (PswDialogView.getView().isVisible()){
            		dialog.setLocationRelativeTo(PswDialogView.getView());
            	}
            	dialog.setVisible(true);

        	} else {
        		processFinished = false;
        	}
            return null;
        }	 
        /**
         * Closes the progress verticalBar. If it wasn't started, nothing happens. 
         */
        public void close() { // is called when work is done
        	processFinished = true; // if the dialog is not already open: do not open
        	if (dialog != null) {
        		dialog.dispose(); 
        		task.cancel(true);
        	}
        }
        /**
         * Sets a new value to show in the progress verticalBar. 
         * If the verticalBar wasn't started, nothing happens. 
         * @param value	 the value to show
         */
        public void setValue(int value){

        	if (threadProgressBar != null){
	        	threadProgressBar.setValue(value);
	        	threadProgressBar.paint(threadProgressBar.getGraphics());
        	} 
        }
    }
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleFileTypePanel();
        }
        return accessibleContext;
    }

    protected class AccessibleFileTypePanel extends AccessibleJPanel {
        //Inherit everything, override nothing.
    }
}