package cologne.eck.all_peas.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.accessibility.AccessibleContext;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;

import cologne.eck.all_peas.data.PeaProperties;

/**
 * 
 *
 * Customized JFileChooser: location is location of parent, not the middle of the screen,
 * icon is not java default icon but PEA icon. 
 * Static method to customize FileChooser (Label...). 
 */

@SuppressWarnings("serial")
public class PeaFileChooser extends JFileChooser implements ActionListener{	

	private static File lastFile;


	public PeaFileChooser (Component comp){
		this.setLocation(comp.getLocationOnScreen());
	}
	
	public PeaFileChooser (Object window) { 
		Component comp = (Component) window;
		this.setLocation(comp.getLocationOnScreen());
	}

/*	public PeaFileChooser(String currentDirectoryPath) {
		super(currentDirectoryPath);
	}

	public PeaFileChooser(File currentDirectory) {
		super(currentDirectory);
	}

	public PeaFileChooser(FileSystemView fsv) {
		super(fsv);
	}

	public PeaFileChooser(File currentDirectory, FileSystemView fsv) {
		super(currentDirectory, fsv);
	}

	public PeaFileChooser(String currentDirectoryPath, FileSystemView fsv) {
		super(currentDirectoryPath, fsv);
	} */
	
    public JDialog createDialog(Object parent)
            throws HeadlessException {
    	
    	JDialog dlg = null;
    	
    	if (parent == null) {
    		dlg = super.createDialog(null);
    	} else {
    		Component parentComponent = (Component) parent;
        	dlg = super.createDialog(parentComponent);
        	
            // location of parent, not middle of scree
        	//System.out.println("location x: " + parent.getLocationOnScreen().getX());
            dlg.setLocation(parentComponent.getLocationOnScreen());
    	}        

        // icon of PEA
        dlg.setIconImage( PswDialogView.getImage() );
        
        JPanel contentPane = (JPanel) dlg.getContentPane();
        contentPane.setBorder(PeaBorderFactory.createBorder(false));
        return dlg;
    }

    /**
     * Adds a title and message on top of the file chooser.
     * Note: This may not work for all Look&Feels. 
     * 
     * @param titleText			the text of the title to set
     * @param instructionText	the text of the message to set on top of the file chooser
     * @param chooser			the file chooser to customize
     */
	public void customizeFileChooser(String titleText, String instructionText, 
			boolean hiddenFilesOption) {
	    // More than customizing the first child panel is error prone
	    // because different Look&Feels build different JFileChoosers. 
	    // For example GTK+  does not work with customization of other panels
		
		if (titleText != null) {
			this.setDialogTitle(titleText);
		}
		
		BorderLayout layout = null;
		try {
			layout = (BorderLayout) this.getLayout();
		} catch (Exception e) {
			// this should not happen
			System.err.println("Missing BorderLayout for JFileChooser");
			return;
		}

		// get the components of the chooser
		Component[]  comps = this.getComponents();
		// get the component of Page_Start if available
		Component pageStartComp = null;
		Object pageStartConstraints = null;

		int compsLen = comps.length;
		for (int i = 0; i < compsLen; i++) {
			Object constraints = layout.getConstraints( comps[i]);
			//System.out.println(i + " constr: " + constraints + ",  " + comps[i].getClass());
			if (constraints.equals(BorderLayout.NORTH)
					|| constraints.equals(BorderLayout.PAGE_START)
					|| constraints.equals(BorderLayout.BEFORE_FIRST_LINE)){ 
				pageStartComp = comps[i];
				pageStartConstraints = constraints;
				break;
			} 
		}		
		
		if (instructionText != null) {
			// build a new panel, add the message 
			// and add the existing panel at page start:
			JPanel northPanel = new JPanel();
			northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.Y_AXIS));
			
			JPanel messagePanel = new JPanel();			
			messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.X_AXIS));
			
			JLabel messageLabel = new JLabel(instructionText);
			messageLabel.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, PswDialogView.getFontsize() + 2));
			
			messagePanel.add(messageLabel);
			messagePanel.add(Box.createHorizontalStrut(10));
			
			northPanel.add(messagePanel);
			northPanel.add(Box.createVerticalStrut(20));

			if (pageStartComp != null){ 
				northPanel.add(pageStartComp);
				this.add(northPanel, pageStartConstraints);
			} else {
				// Look&Feel Motif
				this.add(northPanel, BorderLayout.PAGE_START);
			}
		}
		if (hiddenFilesOption == true) {
			// Add check box for hidden files:
			JCheckBox hideCheck = new JCheckBox("<html>" + PeaProperties.getBundle().getString("show") 
					+ "<br/>" + PeaProperties.getBundle().getString("hidden") 
					+ "<br/>" + PeaProperties.getBundle().getString("files") + "?</html>");
			hideCheck.addActionListener(this);
			hideCheck.setActionCommand("hide");
			this.setAccessory(hideCheck);
		}
		
		if (lastFile != null) {
			this.setSelectedFile(lastFile);
		}
		this.invalidate();
		this.validate();
		this.repaint();
	}
	/**
	 * Remember the selected file to display next time:
	 */
	@Override
    public void approveSelection() {
        lastFile = getSelectedFile();
        super.approveSelection();
    }
	
	/**
	 * Same like JFileChoosers method, parent will
	 * be casted to Component.
	 * 
	 * @param parent	the parent component of this file chooser
	 * 
	 * @return	0 = JFileChooser.APPROVE_OPTION, 
	 * 			1 = JFileChooser.CANCEL_OPTION, 
	 * 			-1 = JFileChooser.ERROR_OPTION if an error occurs or the dialog is dismissed 
	 */
    public int showOpenDialog(Object parent) {
        return super.showOpenDialog((Component) parent);
    }

	@Override
	public void actionPerformed(ActionEvent ape) {
		String command = ape.getActionCommand();
		if (command.equals("hide")){
			JCheckBox hideCheck = (JCheckBox) ape.getSource();
			if (hideCheck.isSelected() == true) {
				this.setFileHidingEnabled(false);
			} else {
				this.setFileHidingEnabled(true);
			}
		}
		
	}
	

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessiblePeaFileChooser();
        }
        return accessibleContext;
    }

    protected class AccessiblePeaFileChooser extends AccessibleJFileChooser {
        //Inherit everything, override nothing.
    }
}
